/*
 *  Copyright (c) 2007 Jiri Benc <jbenc@upir.cz>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <gtk/gtk.h>
#include <hildon-navigator-lib.h>
#include "cmdnav.h"
#include "prefs.h"
#include "run.h"

#define MENU_HEIGHT	13
/* #define DEBUG */

struct cmd_nav_data {
	GtkWidget *button;
	GtkWidget *menu;
	struct cmd *cmds;
	gboolean valid;
	guint prefs_handler;
};

#ifdef DEBUG
#define debug_print	g_print
#else
#define debug_print(...)
#endif

static void menu_invalidate(void *data)
{
	struct cmd_nav_data *d = data;

	debug_print("cmdnav: invalidate\n");

	d->valid = FALSE;
}

void *hildon_navigator_lib_create(void)
{
	struct cmd_nav_data *d;

	if (g_file_test("/media/mmc1/disable-cmd-navigator", G_FILE_TEST_EXISTS) ||
			g_file_test("/media/mmc2/disable-cmd-navigator", G_FILE_TEST_EXISTS))
		return NULL;
	debug_print("cmdnav: create\n");

	d = g_new0(struct cmd_nav_data, 1);

	d->prefs_handler = init_prefs(menu_invalidate, d);

	d->button = gtk_toggle_button_new_with_label("");
	gtk_button_set_image(GTK_BUTTON(d->button),
		gtk_image_new_from_file("/usr/share/icons/hicolor/64x64/apps/terminal.png"));
		//gtk_image_new_from_icon_name("terminal", GTK_ICON_SIZE_DIALOG));
	return d;
}

static void menu_destroy(struct cmd_nav_data *d)
{
	if (d->menu) {
		gtk_menu_shell_deactivate(GTK_MENU_SHELL(d->menu));
		gtk_widget_destroy(d->menu);
		d->menu = NULL;
	}
	if (d->cmds) {
		free_prefs(d->cmds);
		d->cmds = NULL;
	}
}

void hildon_navigator_lib_destroy(void *data)
{
	struct cmd_nav_data *d = data;

	if (!data)
		return;
	debug_print("cmdnav: destroy\n");

	menu_destroy(d);
	/* button is destroyed automatically */

	deinit_prefs(d->prefs_handler);

	g_free(data);
}

static void menu_position(GtkMenu *menu, gint *x, gint *y, gboolean *push_in, gpointer data)
{
	struct cmd_nav_data *d = data;
	GtkRequisition req;
	int screen_dim;

	gtk_widget_size_request(GTK_WIDGET(menu), &req);

	gdk_window_get_origin(d->button->window, x, y);
	*x += d->button->allocation.width - 2;
	if (GTK_WIDGET_NO_WINDOW(d->button)) {
		*x += d->button->allocation.x;
		*y += d->button->allocation.y;

		/* move the menu up if it's too high */
		screen_dim = gdk_screen_get_height(gtk_widget_get_screen(GTK_WIDGET(menu)));
		if (req.height + *y > screen_dim) {
			*y = screen_dim - req.height;
			if (*y < 0)
				*y = 0;
		}
	}

	/* reduce the width of the menu if it's too large */
	screen_dim = gdk_screen_get_width(gtk_widget_get_screen(GTK_WIDGET(menu)));
	if (req.width + *x > screen_dim)
		gtk_widget_set_size_request(GTK_WIDGET(menu), screen_dim - *x, req.height);

	*push_in = FALSE;
}

static void menu_off(GtkMenu *menu, gpointer data)
{
	struct cmd_nav_data *d = data;

	(void)menu;
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(d->button), FALSE);
}

static void run_one_cmd(GtkMenuItem *item, gpointer data)
{
	(void)item;
	run_cmd(data);
}

static void menu_create(struct cmd_nav_data *d)
{
	struct cmd *cur;
	GtkWidget *item, *label;
	int cnt, height, x, y;

	if (d->valid)
		return;

	menu_destroy(d);
	d->menu = gtk_menu_new();
	d->cmds = get_prefs(&cnt);

	d->valid = TRUE;

	x = (cnt + 1 + MENU_HEIGHT - 1) / MENU_HEIGHT;
	height = (cnt + 1 + x - 1) / x;
	x = y = 0;
	for (cur = d->cmds->next; cur; cur = cur->next) {
		item = gtk_menu_item_new_with_label(cur->name);
		g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(run_one_cmd), cur);
		gtk_menu_attach(GTK_MENU(d->menu), item, x, x + 1, y, y + 1);
		if (++y == height) {
			y = 0;
			x++;
		}
	}

	item = gtk_menu_item_new();
	label = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(label), "<span foreground=\"blue\" style=\"italic\">Edit menu...</span>");
	gtk_container_add(GTK_CONTAINER(item), label);
	g_signal_connect(G_OBJECT(item), "activate", G_CALLBACK(run_prefs), NULL);
	gtk_menu_attach(GTK_MENU(d->menu), item, x, x + 1, y, y + 1);

	gtk_widget_show_all(d->menu);

	g_signal_connect(G_OBJECT(d->menu), "hide", G_CALLBACK(menu_off), d);
}

static gboolean menu_on(GtkToggleButton *button, gpointer data)
{
	struct cmd_nav_data *d = data;

	debug_print("cmdnav: toggle\n");

	dezombie();
	if (!gtk_toggle_button_get_active(button))
		return TRUE;

	menu_create(d);
	gtk_menu_popup(GTK_MENU(d->menu), NULL, NULL, menu_position, data,
		1, gtk_get_current_event_time());
	return TRUE;
}

void hildon_navigator_lib_initialize_menu(void *data)
{
	/* According to the documentation, this should be called only
	 * after hildon_navigator_lib_get_button_widget. In reality, this
	 * is not true. */

	struct cmd_nav_data *d = data;

	if (!data)
		return;
	debug_print("cmdnav: initialize_menu\n");

	g_signal_connect(G_OBJECT(d->button), "toggled", G_CALLBACK(menu_on), data);
}

GtkWidget *hildon_navigator_lib_get_button_widget(void *data)
{
	struct cmd_nav_data *d = data;

	if (!data)
		return NULL;
	debug_print("cmdnav: get_button_widget\n");

	gtk_widget_show(d->button);
	return d->button;
}
