/*
 *  Copyright (c) 2007 Jiri Benc <jbenc@upir.cz>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License version 2 as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <string.h>
#include <glib.h>
#include <gtk/gtk.h>
#include <gconf/gconf.h>
#include <gconf/gconf-client.h>
#include "cmdnav.h"
#include "prefs.h"

#define CONF_PREFIX	"/apps/maemo/cmd-navigator"
/* #define DEBUG */

static int refs = 0;
static GConfClient *client;
static gboolean preloaded;

struct invalidate_func {
	void (*invalidate)(void *);
	void *data;
};

/* GConfClientNotifyFunc callback */
static void prefs_changed(GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
{
	struct invalidate_func *f = user_data;

	(void)client; (void)cnxn_id; (void)entry;	/* shut up, gcc */
	if (f->invalidate)
		f->invalidate(f->data);
}

guint init_prefs(void (*invalidate)(void *), void *data)
{
	struct invalidate_func *f;

	if (!refs++) {
		client = gconf_client_get_default();
		gconf_client_add_dir(client, CONF_PREFIX, GCONF_CLIENT_PRELOAD_RECURSIVE, NULL);
		preloaded = TRUE;
	}
	f = g_new(struct invalidate_func, 1);
	f->invalidate = invalidate;
	f->data = data;
	return gconf_client_notify_add(client, CONF_PREFIX, prefs_changed, f, g_free, NULL);
}

void deinit_prefs(guint handler)
{
	gconf_client_notify_remove(client, handler);
	if (!--refs) {
		gconf_client_remove_dir(client, CONF_PREFIX, NULL);
		g_object_unref(G_OBJECT(client));
		client = NULL;
	}
}

void preload_prefs(void)
{
	if (!preloaded) {
		gconf_client_preload(client, CONF_PREFIX, GCONF_CLIENT_PRELOAD_RECURSIVE, NULL);
		preloaded = TRUE;
	}
}

#define MAX_KEY_LEN	(sizeof(CONF_PREFIX) + 32)
struct cmd *_get_prefs(int *dest_cnt)
{
	struct cmd *cmd, *last, *res;
	GSList *keys, *cur_key;
	char key[MAX_KEY_LEN];
	gchar *str;
	int cnt = 0;

	g_assert(preloaded);
	last = res = g_malloc0(sizeof(struct cmd));
	/* in OS2008: last = res = g_slice_alloc0(sizeof(struct cmd)); */
	keys = gconf_client_get_list(client, CONF_PREFIX "/items", GCONF_VALUE_INT, NULL);
	cur_key = keys;
	while (cur_key) {
		snprintf(key, MAX_KEY_LEN, CONF_PREFIX "/%d/name", GPOINTER_TO_INT(cur_key->data));
		str = gconf_client_get_string(client, key, NULL);
		if (!str)
			continue;
		cmd = g_malloc0(sizeof(struct cmd));
		/* in OS2008: cmd = g_slice_alloc0(sizeof(struct cmd)); */
		cmd->id = GPOINTER_TO_INT(cur_key->data);
		cmd->name = str;
		snprintf(key, MAX_KEY_LEN, CONF_PREFIX "/%d/cmd", cmd->id);
		cmd->cmd = gconf_client_get_string(client, key, NULL);
		snprintf(key, MAX_KEY_LEN, CONF_PREFIX "/%d/in-xterm", cmd->id);
		if (gconf_client_get_bool(client, key, NULL))
			cmd->flags |= CMD_FLAG_XTERM;
		snprintf(key, MAX_KEY_LEN, CONF_PREFIX "/%d/as-root", cmd->id);
		if (gconf_client_get_bool(client, key, NULL))
			cmd->flags |= CMD_FLAG_ROOT;
		last->next = cmd;
		last = cmd;

		cnt++;
		cur_key = g_slist_next(cur_key);
	}
	g_slist_free(keys);
	if (dest_cnt)
		*dest_cnt = cnt;
	return res;
}

struct cmd *get_prefs(int *cnt)
{
	struct cmd *res;

	preload_prefs();
	res = _get_prefs(cnt);
	gconf_client_clear_cache(client);
	preloaded = FALSE;
	return res;
}

static void add_to_change_set(GConfChangeSet *set, struct cmd *cur, char *tmp)
{
	snprintf(tmp, MAX_KEY_LEN, CONF_PREFIX "/%d/name", cur->id);
	gconf_change_set_set_string(set, tmp, cur->name);
	snprintf(tmp, MAX_KEY_LEN, CONF_PREFIX "/%d/cmd", cur->id);
	gconf_change_set_set_string(set, tmp, cur->cmd);
	snprintf(tmp, MAX_KEY_LEN, CONF_PREFIX "/%d/in-xterm", cur->id);
	gconf_change_set_set_bool(set, tmp, (cur->flags & CMD_FLAG_XTERM) ? TRUE : FALSE);
	snprintf(tmp, MAX_KEY_LEN, CONF_PREFIX "/%d/as-root", cur->id);
	gconf_change_set_set_bool(set, tmp, (cur->flags & CMD_FLAG_ROOT) ? TRUE : FALSE);
}

void set_prefs(struct cmd *cmds)
{
	 GConfChangeSet *set;
	GSList *keys = NULL;
	struct cmd *cur;
	char key[MAX_KEY_LEN];
	guint max_id;

	g_assert(preloaded);
	set = gconf_change_set_new();
	/* First, walk through the list adding everything which was deleted
	 * and changed (but not added). Also find a greatest id used. */
	max_id = 0;
	for (cur = cmds->next; cur; cur = cur->next) {
		if (!cur->id)
			continue;
		if (cur->id > max_id)
			max_id = cur->id;
		if (cur->flags & CMD_FLAG_DELETED) {
			snprintf(key, MAX_KEY_LEN, CONF_PREFIX "/%d", cur->id);
			/* gconf_change_set_unset(set, key); <- there's no recursive unset? */
			gconf_client_recursive_unset(client, key, 0, NULL);
		} else if (cur->flags & CMD_FLAG_CHANGED)
			add_to_change_set(set, cur, key);
	}
	/* Next, walk through the list again and build an ordered list of
	 * ids. Also add everything which was added. */
	for (cur = cmds->next; cur; cur = cur->next) {
		if (cur->flags & CMD_FLAG_DELETED)
			continue;
		if (!cur->id) {
			cur->id = ++max_id;
			add_to_change_set(set, cur, key);
		}
		keys = g_slist_prepend(keys, GINT_TO_POINTER(cur->id));
	}
	keys = g_slist_reverse(keys);
	gconf_change_set_set_list(set, CONF_PREFIX "/items", GCONF_VALUE_INT, keys);
	/* Commit the changes. */
	gconf_client_commit_change_set(client, set, FALSE, NULL);

	g_slist_free(keys);
	gconf_change_set_unref(set);
}

void free_prefs(struct cmd *cmds)
{
	struct cmd *next;

	while (cmds) {
		next = cmds->next;
		g_free(cmds->name);
		g_free(cmds->cmd);
		g_free(cmds);
		/* in OS2008: g_slice_free1(sizeof(struct cmd), cmds); */
		cmds = next;
	}
}
